require "yaml"
require "spath/version"

module SPath
  DEFAULT_CONFIG_PATH = File.expand_path("~/.spath")

  class ConfigFile

    attr_accessor :path, :shortcuts, :options 

    def initialize(path = DEFAULT_CONFIG_PATH, shortcuts = [], options = {})
      @path = File.expand_path(path)
      @shortcuts = shortcuts
      @options = options
    end

    def <<(shortcut)
      raise ArgumentError "#{shortcut} is not a SPath::Shortcut" \
        unless shortcut.is_a? Shortcut

      # Make room for new shortcuts
      @shortcuts.delete_if { |old_shortcut| shortcut.nickname == old_shortcut.nickname }

      # Append to the array
      @shortcuts << shortcut
    end
    alias_method :add_shortcut, :<<

    def to_hash
      { shortcuts: @shortcuts.map(&:to_hash), options: @options }
    end

    def write!
      File.open(@path, "w") { |f| f.write(to_hash.to_yaml) }
    end

    def self.load(path = DEFAULT_CONFIG_PATH)
      path = File.expand_path(path)

      return new unless File.exists?(File.expand_path(path))

      data = YAML.load(File.read(path))

      # If config file returns empty, use the default
      return new unless data

      shortcuts = data[:shortcuts].map { |sdata| Shortcut[sdata] }
      new(path, shortcuts, data[:options])
    end

    def path_for(nickname)
      raise ArgumentError, "path does not exist for shortcut #{nickname}" unless \
        (shortcut = @shortcuts.select { |s| s.nickname.to_s == nickname }.first)

      shortcut.path
    end

  end

  class Shortcut

     attr_accessor :nickname, :path

     def initialize(nickname, path)
       raise ArgumentError, "#{nickname} is not alphanumeric" \
         if nickname.to_s =~ /[^a-z0-9_]/i
       raise ArgumentError, "nickname is not a string" \
         unless nickname.respond_to? :to_s

       path = File.expand_path(path)
       raise ArgumentError, "path '#{path}' doesn't exist" \
         unless File.exists? path

       @nickname = nickname.to_s.to_sym
       @path = path
     end

     def to_hash
       { nickname: @nickname.to_s, path: @path }
     end

     # Create instance from hash
     def self.[](data)
       new(data[:nickname], data[:path])
     end

  end
end
